require 'spec_helper'

describe Splicer::Dynect::Client do
  let(:client) { Splicer::Dynect::Client.new('customer','username','password') }
  describe '#initialize' do
    subject { client }
    its(:customer) { should eq('customer') }
    its(:username) { should eq('username') }
    its(:password) { should eq('password') }
    its(:base_url) { should eq('https://api-v4.dynect.net/REST') }
  end

  describe '#post' do
    subject { client.post('resource', { data: 'something' }) }

    context 'when Dynect returns with success' do
      before { client.stub(:execute).and_return({}) }
      it { should eq({})}
    end

    context 'when Dynect returns with an error' do
      before { client.stub(:execute).and_raise(Splicer::Errors::RequestError) }
      it 'should raise a Splicer::Errors::RequestError' do
        expect { subject }.to raise_error(Splicer::Errors::RequestError)
      end
    end
  end

  describe '#get' do
    subject { client.get('resource') }

    context 'when Dynect returns with success' do
      before { client.stub(:execute).and_return({}) }
      it { should eq({})}
    end

    context 'when Dynect returns with an error' do
      before { client.stub(:execute).and_raise(Splicer::Errors::RequestError) }
      it 'should raise a Splicer::Errors::RequestError' do
        expect { subject }.to raise_error(Splicer::Errors::RequestError)
      end
    end
  end

  describe '#put' do
    subject { client.put('resource', { data: 'something' }) }

    context 'when Dynect returns with success' do
      before { client.stub(:execute).and_return({}) }
      it { should eq({})}
    end

    context 'when Dynect returns with an error' do
      before { client.stub(:execute).and_raise(Splicer::Errors::RequestError) }
      it 'should raise a Splicer::Errors::RequestError' do
        expect { subject }.to raise_error(Splicer::Errors::RequestError)
      end
    end
  end

  describe '#delete' do
    subject { client.delete('resource') }

    context 'when Dynect returns with success' do
      before { client.stub(:execute).and_return({}) }
      it { should eq({})}
    end

    context 'when Dynect returns with an error' do
      before { client.stub(:execute).and_raise(Splicer::Errors::RequestError) }
      it 'should raise a Splicer::Errors::RequestError' do
        expect { subject }.to raise_error(Splicer::Errors::RequestError)
      end
    end
  end

  describe '#login' do
    subject { client.login }

    context 'when logged in' do
      before { client.stub(:logged_in?).and_return(true) }
      it { should eq(true) }
    end

    context 'when not logged in' do
      let(:data) { {'data' => {'token' => 'blah'}} }
      before { client.stub(:post).and_return(data) }

      it { should eq(true) }
      it 'should set the Auth-Token' do
        subject
        expect(client.headers['Auth-Token']).to eq('blah')
      end
    end
  end

  describe '#noop' do
    subject { client.noop }

    context 'when logged in' do
      before do
        client.stub(:logged_in?).and_return(true)
        client.stub(:put).and_return({})
      end
      it { should eq(true) }
    end

    context 'when not logged in' do
      before { client.stub(:logged_in?).and_return(false) }
      it { should eq(false) }
    end
  end

  describe '#logout' do
    subject { client.logout }

    context 'when logged in' do
      before do
        client.headers['Auth-Token'] = 'blah'
        client.stub(:delete).and_return({})
      end
      it { should eq(true) }
      it 'should unset the Auth-Token' do
        subject
        expect(client.headers['Auth-Token']).to eq(nil)
      end
    end

    context 'when not logged in' do
      before { client.stub(:logged_in?).and_return(false) }
      it { should eq(false) }
    end
  end
end
